@@ -5,6 +5,8 @@ Modified from werkzeug.debug.tbtools
 """
 
 # -- stdlib --
+from types import TracebackType
+from typing import Any, Optional, Tuple, Type, Union, cast
 import codecs
 import inspect
 import os
@@ -14,7 +16,6 @@ import sysconfig
 import traceback
 
 # -- third party --
-from werkzeug._compat import range_type, reraise, string_types, text_type, to_native, to_unicode
 from werkzeug.filesystem import get_filesystem_encoding
 from werkzeug.utils import cached_property
 
@@ -24,8 +25,25 @@ from .console import Console
 # -- code --
 _coding_re = re.compile(br"coding[:=]\s*([-\w.]+)")
 _line_re = re.compile(br"^(.*?)$", re.MULTILINE)
-_funcdef_re = re.compile(r"^(\s*def\s)|(.*(?<!\w)lambda(:|\s))|^(\s*@)")
-UTF8_COOKIE = b"\xef\xbb\xbf"
+_default_encoding = sys.getdefaultencoding()
+
+def _to_str(
+    x: Optional[Any],
+    charset: Optional[str] = _default_encoding,
+    errors: str = "strict",
+    allow_none_charset: bool = False,
+) -> Optional[Union[str, bytes]]:
+    if x is None or isinstance(x, str):
+        return x
+
+    if not isinstance(x, (bytes, bytearray)):
+        return str(x)
+
+    if charset is None:
+        if allow_none_charset:
+            return x
+
+    return x.decode(charset, errors)  # type: ignore
 
 system_exceptions = (SystemExit, KeyboardInterrupt)
 try:
@@ -34,6 +52,37 @@ except NameError:
     pass
 
 
+def get_current_traceback(
+    ignore_system_exceptions: bool = False,
+    show_hidden_frames: bool = False,
+    skip: int = 0,
+) -> "Traceback":
+    """Get the current exception info as `Traceback` object.  Per default
+    calling this method will reraise system exceptions such as generator exit,
+    system exit or others.  This behavior can be disabled by passing `False`
+    to the function as first parameter.
+    """
+    info = cast(
+        Tuple[Type[BaseException], BaseException, TracebackType], sys.exc_info()
+    )
+    exc_type, exc_value, tb = info
+
+    if ignore_system_exceptions and exc_type in {
+        SystemExit,
+        KeyboardInterrupt,
+        GeneratorExit,
+    }:
+        raise
+    for _ in range(skip):
+        if tb.tb_next is None:
+            break
+        tb = tb.tb_next
+    tb = Traceback(exc_type, exc_value, tb)
+    if not show_hidden_frames:
+        tb.filter_hidden_frames()
+    return tb
+
+
 class Traceback(object):
     """Wraps a traceback."""
 
@@ -76,8 +125,8 @@ class Traceback(object):
         """Log the ASCII traceback into a file object."""
         if logfile is None:
             logfile = sys.stderr
-        tb = self.plaintext.rstrip() + u"\n"
-        logfile.write(to_native(tb, "utf-8", "replace"))
+        tb = f"{self.plaintext.rstrip()}\n"
+        logfile.write(tb)
 
     def render_groups(self, include_title=True):
         """Render the traceback for the interactive console."""
@@ -161,7 +210,7 @@ class Group(object):
         """String representation of the exception."""
         buf = traceback.format_exception_only(self.exc_type, self.exc_value)
         rv = "".join(buf).strip()
-        return to_unicode(rv, "utf-8", "replace")
+        return _to_str(rv, "utf-8", "replace")
 
     def render(self):
         relation = 'root'
@@ -210,8 +259,8 @@ class Frame(object):
         else:
             realpath = fn
 
-        self.filename = to_unicode(fn, get_filesystem_encoding())
-        self.realpath = to_unicode(realpath, get_filesystem_encoding())
+        self.filename = _to_str(fn, get_filesystem_encoding())
+        self.realpath = _to_str(realpath, get_filesystem_encoding())
         self.abbrname = self.abbr_filename(self.realpath) or self.filename
         self.module = self.globals.get("__name__", self.locals.get("__name__"))
         self.loader = self.globals.get("__loader__", self.locals.get("__loader__"))
@@ -289,9 +338,7 @@ class Frame(object):
 
     def eval(self, code, mode="single"):
         """Evaluate code in the context of the frame."""
-        if isinstance(code, string_types):
-            if isinstance(code, text_type):  # noqa
-                code = UTF8_COOKIE + code.encode("utf-8")
+        if isinstance(code, str):
             code = compile(code, "<interactive>", mode)
         return eval(code, self.globals, self.locals)
 
@@ -313,33 +360,29 @@ class Frame(object):
 
         if source is None:
             try:
-                with open(
-                    to_native(self.filename, get_filesystem_encoding()), mode="rb"
-                ) as f:
+                with open(self.filename, mode="rb") as f:
                     source = f.read()
             except IOError:
                 return []
 
-        # already unicode?  return right away
-        if isinstance(source, text_type):
+        # already str?  return right away
+        if isinstance(source, str):
             return source.splitlines()
 
-        # yes. it should be ascii, but we don't want to reject too many
-        # characters in the debugger if something breaks
         charset = "utf-8"
-        if source.startswith(UTF8_COOKIE):
+        if source.startswith(codecs.BOM_UTF8):
             source = source[3:]
         else:
             for idx, match in enumerate(_line_re.finditer(source)):
-                match = _coding_re.search(match.group())
-                if match is not None:
-                    charset = match.group(1)
+                coding_match = _coding_re.search(match.group())
+                if coding_match is not None:
+                    charset = coding_match.group(1).decode("utf-8")
                     break
                 if idx > 1:
                     break
 
         # on broken cookies we fall back to utf-8 too
-        charset = to_native(charset)
+        charset = _to_str(charset)
         try:
             codecs.lookup(charset)
         except LookupError:
