@@ -30,6 +30,13 @@ def _strip_bytes(data):
     return data
 
 
+def get_request_id():
+    try:
+        return int(request.headers['X-Openbayes-Serving-Request-Id'])
+    except Exception:
+        return 0
+
+
 def emit_event(name, data):
     # Enconding should happen before logging,
     # to make it crash in development environments
@@ -47,9 +54,8 @@ def emit_event(name, data):
         log.info('记录事件：%s -> %s', name, data)
         return
 
-    try:
-        req_id = int(request.headers['X-Openbayes-Serving-Request-Id'])
-    except Exception:
+    req_id = get_request_id()
+    if req_id == 0:
         log.error("无法获取 Request Id，不会记录当前事件。")
         return
 
